﻿// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "LMCustomMovementModes.h"
#include "GameFramework/CharacterMovementComponent.h"
#include "LMMovementComponent.generated.h"


class ALMPlayer;

// See https://www.youtube.com/playlist?list=PLXJlkahwiwPmeABEhjwIALvxRSZkzoQpk
UCLASS(ClassGroup=(Custom), meta=(BlueprintSpawnableComponent))
class LEGUMEMIX_API ULMMovementComponent : public UCharacterMovementComponent
{
	GENERATED_BODY()

	class FSavedMove_Legumix : public FSavedMove_Character
	{
	public:
		uint8 Saved_bPrevWantsToCrouch:1;

		FSavedMove_Legumix();
		
		virtual bool CanCombineWith(const FSavedMovePtr& NewMove, ACharacter* InCharacter, float MaxDelta) const override;
		virtual void Clear() override;
		virtual uint8 GetCompressedFlags() const override;
		virtual void SetMoveFor(ACharacter* C, float InDeltaTime, FVector const& NewAccel, class FNetworkPredictionData_Client_Character& ClientData) override;
		virtual void PrepMoveFor(ACharacter* C) override;
	};

	class FNetworkPredictionData_Client_Legumix : public FNetworkPredictionData_Client_Character
	{
	public:
		FNetworkPredictionData_Client_Legumix(const UCharacterMovementComponent& ClientMovement);
		typedef FNetworkPredictionData_Client_Character Super;
		virtual FSavedMovePtr AllocateNewMove() override;
	};

	bool Safe_bPrevWantsToCrouch = false;

public:
	ULMMovementComponent();
	
protected:
	virtual void InitializeComponent() override;

public:
	virtual FNetworkPredictionData_Client* GetPredictionData_Client() const override;
	virtual bool IsMovingOnGround() const override;
	virtual bool CanCrouchInCurrentState() const override;

protected:
	virtual void UpdateFromCompressedFlags(uint8 Flags) override;

public:
	virtual void UpdateCharacterStateBeforeMovement(float DeltaSeconds) override;

protected:
	virtual void OnMovementUpdated(float DeltaSeconds, const FVector& OldLocation, const FVector& OldVelocity) override;
	virtual void PhysCustom(float DeltaTime, int32 Iterations) override;


public:
	UFUNCTION(BlueprintPure)
	bool IsCustomMovementMode(ECustomMovementModes InCustomMovementMode) const;
	
public:
	UPROPERTY(Transient)
	TObjectPtr<ALMPlayer> PlayerCharacterOwner;
	
	// Minimum speed require to slide
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category="Legumix|Slide")
	float SlideMinSpeed = 350.f;
	// Boost in velocity when entering a slide
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category="Legumix|Slide")
	float SlideEnterImpulse = 500.f;
	// Amount of force applied to the player to force it on the ground, also affects the speed of a slide on a slope.
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category="Legumix|Slide")
	float SlideGravityForce = 5000.f;
	// How fast you loose velocity as you slide.
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category="Legumix|Slide")
	float SlideFriction = 1.3f;

private:
	void EnterSlide();
	void ExitSlide();
	void PhysSlide(float DeltaTime, int32 Iterations);
	bool GetSlideSurface(FHitResult& Hit) const; 

	UFUNCTION(BlueprintCallable, Category="Legumix|Slide")
	void CrouchPressed();
};
